package com.v1_4.BD890857AD06CA0258B9A520.com;
import java.util.ArrayList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONObject;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.drawable.Drawable;
import android.location.LocationListener;
import android.location.LocationManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.TextView;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.MyLocationOverlay;
import com.google.android.maps.Overlay;
import com.google.android.maps.OverlayItem;


public class Screen_LocationMap extends MapActivity{

	
	public AppDelegate appDelegate;
	public DownloadThread downloadThread;
	public AlertDialog myAlert;
	public ProgressDialog myProgressBox;

	//data..
	public String thisActivityName = "Act_LocationMap";
	public String appGuid = "";
	public String appApiKey = "";
	public String screenGuid = "";
	public String saveAsFileName = "";
	public String remoteDataCommand = "";
	public String JSONData = "";
		
	//map view
	public MapView myMap = null;
	public int showUserLocation = 0;
	public LocationManager myLocationManager;
	public ArrayList<Obj_MapLocation> myLocationObjects;
	public int droppedDevicePin = 0;
	public int selectedIndex = -1;
	
	//overlays
	List<Overlay> mapOverlays;
	Drawable locationPin;
	MyLocationsOverlay itemizedOverlay;	
	
	//for zooming span
	public ArrayList<GeoPoint> myPoints;
  	 
	//onCreate
	public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.screen_locationmap);

        //appDelegate holds current screen
	    this.appDelegate = (AppDelegate) this.getApplication();
        appGuid = appDelegate.currentApp.guid;
        appApiKey = appDelegate.currentApp.apiKey;
    	screenGuid = appDelegate.currentScreen.screenGuid;
    	
        //back button..
        ImageButton btnBack = (ImageButton) findViewById(R.id.btnBack);
        btnBack.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
            	finish();
           }
        });
        //info button
        ImageButton btnInfo = (ImageButton) findViewById(R.id.btnInfo);
        btnInfo.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                if (v.isSelected()){
                    v.setSelected(false);
                } else {
                    v.setSelected(true);
                }
            	showInfo();
           }
        });    	
    	
 	
    	//init map
		myMap = (MapView)findViewById(R.id.mapView);
		myMap.setBuiltInZoomControls(true);	

		//command depends on screen type, no data download needed for single locations
    	if(appDelegate.currentScreen.screenType.equals("screen_locationMap")){
    		saveAsFileName = "";
    		remoteDataCommand = "";
			((TextView)findViewById(R.id.myTitle)).setText("Location");
    	}else{
    		saveAsFileName = appDelegate.currentScreen.screenGuid + "_mapLocs.txt";
    		remoteDataCommand = "multiLocationMapViewController";
            ((TextView)findViewById(R.id.myTitle)).setText(appDelegate.currentScreen.screenTitle);
    	}
	        
    }
	
	//onStart
	@Override 
	protected void onStart() {
		super.onStart();
			
		//if we have a "saveAsFileName" we need multiple location data..
		if(saveAsFileName.length() > 1 && remoteDataCommand.length() > 1){
		
			//local or remote JSON text...
			if(appDelegate.fileExists(saveAsFileName)){
				//Log.i("ZZ", thisActivityName + ":onStart local data exists");      
				JSONData = appDelegate.getLocalText(saveAsFileName);
				parseData(JSONData);
			}else{
				//parse called after when done
				//Log.i("ZZ", thisActivityName + ":onStart downloading data");      
				downloadData();			
			}
		}else{
			parseData("locationMap");			
		}
	}

	
	//onStop
	@Override 
	protected void onStop(){
		super.onStop();
		if(downloadThread != null){
			boolean retry = true;
			downloadThread.setThreadRunning(false);
			while(retry){
				try{
					downloadThread.join();
					retry = false;
				}catch (Exception je){
					//Log.i("ZZ", thisActivityName + ":onStop ERROR: " + je.getMessage());      
				}
			}
		}
	}
	
	//onDestroy
	protected void onDestroy(){
		super.onDestroy();
	}
	
	
	@Override
	public void onResume() {
		super.onResume();
	}		

	@Override
	public void onPause() {
		super.onPause();
	}		
	
	
	//download data
	public void downloadData(){
		
		//show progress
		showProgress("Loading...", "Adding points to the map.");
		
		//single location screens don't need a download...
		if(saveAsFileName.length() > 1 && remoteDataCommand.length() > 1){
		
			//build URL
			String tmpURL = appDelegate.currentApp.dataUrl;
			tmpURL += "?appGuid=" + appGuid;
			tmpURL += "&appApiKey=" + appApiKey;	
			tmpURL += "&screenGuid=" + screenGuid;
			tmpURL += "&command=" + remoteDataCommand;	
			
				downloadThread = new DownloadThread();
				downloadThread.setDownloadURL(tmpURL);
				downloadThread.setSaveAsFileName(saveAsFileName);
				downloadThread.setDownloadType("text");
				downloadThread.setThreadRunning(true);
				downloadThread.start();
		}else{
			parseData("locationMap");			
		}
			
	}
	
	//parse data..
	public void parseData(String theJSONString){
		//parse JSON string
		//Log.i("ZZ", thisActivityName + ":parseData: " + theJSONString);   
		
		//init location objects..
        myLocationObjects = new ArrayList<Obj_MapLocation>();
		
		//singleLocaiton map...
    	if(appDelegate.currentScreen.screenType.equals("screen_locationMap")){
    		try{
    		
	   			
	   			JSONObject raw = new JSONObject(appDelegate.currentScreen.jsonScreenOptions);
		
				//one location object...
				Obj_MapLocation tmpLocationObj = new Obj_MapLocation(raw.getString("annotationTitle"), raw.getString("annotationSubTitle"), 0);
				tmpLocationObj.point = getPoint(Double.parseDouble(raw.getString("lattitude")), Double.parseDouble(raw.getString("longitude")));
				tmpLocationObj.latitude = raw.getString("lattitude");
				tmpLocationObj.longitude = raw.getString("longitude");
				tmpLocationObj.detailsTitle = "";
				tmpLocationObj.detailsURL = ""; 
				myLocationObjects.add(tmpLocationObj);
	    	
				
    		}catch (Exception je){
	    		//showAlert("Data Format Error", "There was a problem reading data associated with this app.");
	    	}	
    		
    	}else{
		
    		//multiple location map...
  	    	try{
  	    		Log.i("ZZ", thisActivityName + ":parseData multi locations");   

	      		
  	    		//create json objects from data string...
	    		JSONObject raw = new JSONObject(theJSONString);
	            JSONObject results =  raw.getJSONObject("results");
	            JSONArray locations =  results.getJSONArray("locations");
	            
	            //loop    
	            int i = 0;
	            for (i = 0; i < locations.length(); i++){
	            	JSONObject tmpJson = locations.getJSONObject(i);

	            	//one location object...
					Obj_MapLocation tmpLocationObj = new Obj_MapLocation(tmpJson.getString("title"), tmpJson.getString("subTitle"), 0);
					tmpLocationObj.point = getPoint(Double.parseDouble(tmpJson.getString("latitude")), Double.parseDouble(tmpJson.getString("longitude")));
					tmpLocationObj.latitude = tmpJson.getString("latitude");
					tmpLocationObj.longitude = tmpJson.getString("longitude");
					tmpLocationObj.detailsTitle = tmpJson.getString("detailsTitle");
					tmpLocationObj.detailsURL = tmpJson.getString("detailsUrl"); 
					myLocationObjects.add(tmpLocationObj);

	            	
	            
	            }//end for
	          
	    	}catch (Exception je){
	    		//showAlert("Data Format Error", "There was a problem reading data associated with this app.");
	    	}
    	
    	}
    	
		//show pins..
		showMapPins();
		//Log.i("ZZ", thisActivityName + ":myLocationObjets count: " + myLocationObjects.size());   
	}

	/////////////////////////////////////////////////////
	//done downloading data..

	Handler downloadTextHandler = new Handler(){
		@Override public void handleMessage(Message msg){
			if(JSONData.length() < 1){
				hideProgress();
				showAlert("Error Downloading", "Please check your internet connection then try again.");
			}else{
				parseData(JSONData);
			}
		}
	};		
	
	//download class in new thread / class
	public class DownloadThread extends Thread{
		 boolean threadRunning = false;
		 String downloadURL = "";
		 String saveAsFileName = "";
		 String downloadType = "";
		 void setThreadRunning(boolean bolRunning){
			 threadRunning = bolRunning;
		 }		
		 void setDownloadURL(String theURL){
			 downloadURL = theURL;
		 }
		 void setSaveAsFileName(String theFileName){
			 saveAsFileName = theFileName;
		 }
		 void setDownloadType(String imageOrText){
			 downloadType = imageOrText;		 
		 }
    	 @Override 
    	 public void run(){
			
			if(downloadType == "text"){
				JSONData = appDelegate.downloadText(downloadURL);
				appDelegate.saveText(saveAsFileName, JSONData);
				downloadTextHandler.sendMessage(downloadTextHandler.obtainMessage());
				this.setThreadRunning(false);
			}
    	 }
	}	
	
	
	//show map pins...
	public void showMapPins(){
		
		//init points array
	    myPoints = new ArrayList<GeoPoint>();

	    //clear map..
        mapOverlays = myMap.getOverlays();
        locationPin = this.getResources().getDrawable(R.drawable.map_marker_red);
       	itemizedOverlay = new MyLocationsOverlay(locationPin);        
        myMap.invalidate();
        
        //loop data.. (track max / min latitude and longitude so we can zoom properley.
        
		int i = 0;
		for (i = 0; i < myLocationObjects.size(); i++){
			Obj_MapLocation thisLocation = myLocationObjects.get(i);
			
			//overlay item..
			OverlayItem overlayitem = new OverlayItem(thisLocation.point, thisLocation.annotationTitle, thisLocation.annotationSubTitle);			
	        Drawable pinGraphic = this.getResources().getDrawable(R.drawable.map_marker_red);
	        overlayitem.setState(pinGraphic, 0);
	        itemizedOverlay.addOverlay(overlayitem, pinGraphic);
			
			//add point to list
			myPoints.add(thisLocation.point);
				
	    }//end for..
		
		//add device location if we have it..
		if(appDelegate.currentDevice.deviceLatitude.length() > 4){
				
			Obj_MapLocation deviceLoc = new Obj_MapLocation("", "", 1);
				deviceLoc.latitude = appDelegate.currentDevice.deviceLatitude;
				deviceLoc.longitude = appDelegate.currentDevice.deviceLongitude;
				deviceLoc.annotationTitle = "";
				deviceLoc.annotationSubTitle = "";
				deviceLoc.point = getPoint(Double.parseDouble(deviceLoc.latitude), Double.parseDouble(deviceLoc.longitude));
				deviceLoc.detailsTitle = "";
				deviceLoc.detailsURL = ""; 
				
   				OverlayItem overlayitem = new OverlayItem(deviceLoc.point, deviceLoc.annotationTitle, deviceLoc.annotationSubTitle);			
   		        Drawable pinGraphic = this.getResources().getDrawable(R.drawable.youarehere);
   
   		        overlayitem.setState(pinGraphic, 0);
   		        itemizedOverlay.addOverlay(overlayitem, pinGraphic);
   				myLocationObjects.add(deviceLoc);
   				myPoints.add(deviceLoc.point);
   		}	
        	            	
		//add itemizedOverlay array to map
		mapOverlays.add(itemizedOverlay);
		
		//rebuild map.
        myMap.invalidate();  
        setMapBoundsToPois(myPoints, 0.1, 0.1);
        hideProgress();
        
	}
	
	public void setMapBoundsToPois(List<GeoPoint> items, double hpadding, double vpadding){
	    MapController mapController = myMap.getController();
	    // If there is only on one result directly animate to that location
	    if (items.size() == 1) { // animate to the location
	        mapController.animateTo(items.get(0));
	    } else {
	        // find the lat, lon span
	        int minLatitude = Integer.MAX_VALUE;
	        int maxLatitude = Integer.MIN_VALUE;
	        int minLongitude = Integer.MAX_VALUE;
	        int maxLongitude = Integer.MIN_VALUE;

	        // Find the boundaries of the item set
	        for (GeoPoint item : items) {
	            int lat = item.getLatitudeE6(); int lon = item.getLongitudeE6();

	            maxLatitude = Math.max(lat, maxLatitude);
	            minLatitude = Math.min(lat, minLatitude);
	            maxLongitude = Math.max(lon, maxLongitude);
	            minLongitude = Math.min(lon, minLongitude);
	        }

	        // leave some padding from corners
	        // such as 0.1 for hpadding and 0.2 for vpadding
	        maxLatitude = maxLatitude + (int)((maxLatitude-minLatitude)*hpadding);
	        minLatitude = minLatitude - (int)((maxLatitude-minLatitude)*hpadding);

	        maxLongitude = maxLongitude + (int)((maxLongitude-minLongitude)*vpadding);
	        minLongitude = minLongitude - (int)((maxLongitude-minLongitude)*vpadding);

	        // Calculate the lat, lon spans from the given pois and zoom
	        mapController.zoomToSpan(Math.abs(maxLatitude - minLatitude), Math.abs(maxLongitude - minLongitude));

	        // Animate to the center of the cluster of points
	        mapController.animateTo(new GeoPoint((maxLatitude + minLatitude) / 2, (maxLongitude + minLongitude) / 2));
	    }
	    
	    //invalidate to "refresh" map
        myMap.invalidate();
	} // end of the method

	//remove locations..
	public void removeLocations(){
        List<Overlay> listOfOverlays = myMap.getOverlays();
        listOfOverlays.clear();
        myMap.invalidate();
    }
	
	
	//location update methods
 	@Override
	protected boolean isRouteDisplayed() {
		return(false);
	}
 	
 	//shows map type
 	public void showMapType(String mapType){
 		if(mapType == "satelliteView"){
 	        myMap.setSatellite(true);
 		}
 		if(mapType == "streetView"){
 	        myMap.setSatellite(false);
 		}
  	}
 	
 	//creates point
	private GeoPoint getPoint(double lat, double lon) {
		return(new GeoPoint((int)(lat*1000000.0),(int)(lon*1000000.0)));
	}
	
	//refresh
	public void refresh(){
	   	downloadData();
	}
		
	
	//////////////////////////////////////////////////////////////////
	//location overlay manager..
	public class MyLocationsOverlay extends ItemizedOverlay {
		private ArrayList<OverlayItem> mOverlays = new ArrayList<OverlayItem>();
		
		public MyLocationsOverlay(Drawable defaultMarker) {
			super(boundCenterBottom(defaultMarker));
		}

		public void addOverlay(OverlayItem overlay, Drawable theMarker) {
			this.boundCenterBottom(theMarker);
			overlay.setMarker(theMarker);
			mOverlays.add(overlay);
		    populate();
		}	
		
		@Override
		protected OverlayItem createItem(int i) {
		  return mOverlays.get(i);
		}

		@Override
		public int size() {
			return mOverlays.size();
		}

		@Override
		protected boolean onTap(int i) {
			
			//get Obj_MapLocation for selected index..
			Obj_MapLocation tappedLocation = (Obj_MapLocation) myLocationObjects.get(i);
			selectedIndex = i;
			
			//bubble depends on which type of pin was tapped. Device's location or regular location
			if(tappedLocation.isDeviceLocation == 1){
				showAlert("You are here", "As reported by your device.");
			}else{
				if(tappedLocation.detailsURL.length() > 3){
					showLocationBubble(tappedLocation.annotationTitle, tappedLocation.annotationSubTitle, "details");
				}else{
					showLocationBubble(tappedLocation.annotationTitle, tappedLocation.annotationSubTitle, "directions");
				}
			}
			
			return(true);
		}	
	}
	//////////////////////////////////////////////////////////////////
	
	
	////////////////////////////////////////////////////////////////////////
	//show / hide progress
	public void showProgress(String theTitle, String theMessage){
		if(theTitle == "") theTitle = "No Progress Title?";
		if(theMessage == "") theMessage = "No progress message?";
		myProgressBox = ProgressDialog.show(this, theTitle, theMessage, true);
	}
	public void hideProgress(){
		if(myProgressBox != null){
			myProgressBox.dismiss();
		}
	}
	
	//show alert message
	public void showAlert(String theTitle, String theMessage) {
		if(theTitle == "") theTitle = "No Alert Title?";
		if(theMessage == "") theMessage = "No alert message?";
		myAlert = new AlertDialog.Builder(this).create();
		myAlert.setTitle(theTitle);
		myAlert.setMessage(theMessage);
		myAlert.setIcon(R.drawable.marker);
		myAlert.setCancelable(false);
		myAlert.setButton("Close", new DialogInterface.OnClickListener() {
	      public void onClick(DialogInterface dialog, int which) {
	        myAlert.dismiss();
	    } }); 
		myAlert.show();
	}		
	
	//location bubble..
	public void showLocationBubble(String theTitle, String theMessage, String detailsOrDirections) {
		myAlert = new AlertDialog.Builder(this).create();
		myAlert.setTitle(theTitle);
		myAlert.setMessage(theMessage);
		myAlert.setIcon(R.drawable.marker);
		myAlert.setCancelable(false);
		myAlert.setButton2("Close", new DialogInterface.OnClickListener() {
		      public void onClick(DialogInterface dialog, int which) {
		    	myAlert.dismiss();
		    } }); 
		
		//buttons depend..
		if(detailsOrDirections == "details"){
			myAlert.setButton("More...", new DialogInterface.OnClickListener() {
			      public void onClick(DialogInterface dialog, int which) {
			        myAlert.dismiss();
			        showMoreDetails();
			 } }); 
		}
		if(detailsOrDirections == "directions"){
			myAlert.setButton("Driving Directions", new DialogInterface.OnClickListener() {
			      public void onClick(DialogInterface dialog, int which) {
			        myAlert.dismiss();
			        showDirections();
			} }); 
		}
		
		
		myAlert.show();
	}
	
	//shows directions..
	public void showDirections(){
		if(selectedIndex > -1){
			
			//we must have a device location...
			if(appDelegate.currentDevice.deviceLatitude.length() > 3){
			
				//currentDevice.location > Obj_MapLocation.location
				Obj_MapLocation tappedLocation = (Obj_MapLocation) myLocationObjects.get(selectedIndex);
				Uri uri = Uri.parse("http://maps.google.com/maps?saddr=" + appDelegate.currentDevice.deviceLatitude + "," + appDelegate.currentDevice.deviceLongitude + "&daddr=" + tappedLocation.latitude + "," + tappedLocation.longitude);
				Intent intent = new Intent(android.content.Intent.ACTION_VIEW, uri);
				startActivity(intent);
			
			}else{
				showAlert("We can't find you?","Driving directions cannot be determined because your device is not reporting it's location");
			}

		}
	}
	
	//shows more details in a web-view if detailsURL provided...
	public void showMoreDetails(){
		
		Obj_MapLocation tappedLocation = (Obj_MapLocation) myLocationObjects.get(selectedIndex);
		
		//create an Obj_Screen from RSSStory data..
		Obj_Screen tmpScreen = new Obj_Screen("0000", "0000");
		tmpScreen.setScreenType("screen_url");
		tmpScreen.setScreenTitle(tappedLocation.detailsTitle);
        
        //create JSON from RSSStory data
        String tmpKeys = "{";
        	tmpKeys += "\"title\":\"More...\",";
        	tmpKeys += "\"url\":\"" + tappedLocation.detailsURL + "\"";
        	tmpKeys += "}";
        	tmpScreen.setJsonScreenOptions(tmpKeys);
        appDelegate.currentScreen = tmpScreen;
		
        //custom URL intent will use screen remembered in appDelegate
        Intent theIntent = new Intent(this, Screen_CustomURL.class);
		startActivity(theIntent);
		
		
	}
	
	//show info
	public void showInfo(){
      	Intent theIntent = new Intent(this, Screen_About.class);
        startActivity(theIntent);
	}	

	/////////////////////////////////////////////////////
	//options menu
	@Override 
	public boolean onPrepareOptionsMenu(Menu menu) { 
		super.onPrepareOptionsMenu(menu); 
		
		 //set up dialog
        final Dialog dialog = new Dialog(this);
        dialog.setContentView(R.layout.menu_locationmap);
        dialog.setTitle("Screen Options");
        
        //terrain ..
        Button btnTerrain = (Button) dialog.findViewById(R.id.btnSatellite);
        btnTerrain.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
            	dialog.cancel();
                showMapType("satelliteView");
            }
        });

        //street ..
        Button btnStreet = (Button) dialog.findViewById(R.id.btnStreet);
        btnStreet.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
            	dialog.cancel();
                showMapType("streetView");
            }
        });       
        
       //refresh ..
        Button btnRefresh = (Button) dialog.findViewById(R.id.btnRefresh);
        btnRefresh.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
            	dialog.cancel();
                refresh();
            }
        });

        //cancel...
        Button button = (Button) dialog.findViewById(R.id.btnCancel);
        button.setOnClickListener(new OnClickListener(){
            public void onClick(View v) {
               dialog.cancel();
            }
        });        
        
        //show
        dialog.show();
		return true;
		
	} 
	//end menu
	/////////////////////////////////////////////////////
		
	
	
	
	
	
}






